// Parallel Simulated Annealing
// for the Set-partitioning Problem
// Mateusz Styrczula, Michał Kraszewski
// 13.01.2009


#define _CRT_RAND_S

#include <iostream>
#include <sstream>
#include <cstdlib>
#include "Main.h"
#include "Solution.h"



using namespace std;
using namespace PSASP;



// ...
int main (int argc, char *argv[]) {

    bool prop_results = true;
    int thread_count = THREAD_COUNT_INIT;
    int map_size = MAP_SIZE_INIT;
    bool map_filename_passed = false;

    // primitively parse program arguments  (any reasonable error handling is completely ommited...)
    if (argc == 1) {
        cout << "Program usage:  " << argv[0] << " PROP_RESULTS THREAD_COUNT [MAPSIZE] [MAPFILE]" << endl;
        cout << "                PROP_RESULTS = 0  --  do not propagate results in chain topology" << endl;        
        cout << "                PROP_RESULTS > 0  --  propagate results in chain topology" << endl;
        cout << "                    THREAD_COUNT  --  number of threads to involve (min.: 1, max.: " << THREAD_COUNT_MAX << ")" << endl;
        cout << "                         MAPSIZE  --  number of customers in map (min.: 3, max.: " << MAX_MAP_SIZE << ")" << endl;
        cout << "                         MAPFILE  --  name of file with customers coordinates" << endl; 
        return 1;
    }
    if (argc > 1) {
        stringstream ss (argv[1]);
        int pr = 0;
        ss >> pr;
        if (pr == 0) {
            prop_results = false;
            cout << "Propagating results in chain topology is OFF" << endl;
        } else {
            prop_results = true;
            cout << "Propagating results in chain topology is ON" << endl;
        }
    }
    if (argc > 2) {
        stringstream ss (argv[2]);
        ss >> thread_count;
        if (thread_count < 1  ||  thread_count > THREAD_COUNT_MAX) {
            cout << "THREAD_COUNT value must be in 1.." << THREAD_COUNT_MAX << " range!" << endl;
            return 1;
        }
    }
    if (argc > 3) {
        stringstream ss (argv[3]);
        ss >> map_size;
        if (map_size < 3  ||  map_size > MAX_MAP_SIZE) {
            cout << "MAP_SIZE is completely irrational!" << endl;
            return 1;
        }
    }
    if (argc == 5) {
        map_filename_passed = true;
    }

    // init pseudo-random number generator
    std::srand (time (NULL));
    
    // create "Main" object, run it's body and "pretty-print" solution
    Main *m = NULL;
    if (map_filename_passed)
        m = new Main (map_size, argv[4], thread_count, prop_results);
    else
        m = new Main (map_size, "", thread_count, prop_results);
    m->body ();
    m->best_solution.print ();
    delete m;

    return 0;
}
