// Parallel Simulated Annealing
// for the Set-partitioning Problem
// Mateusz Styrczula, Michał Kraszewski
// 13.01.2009



#define _CRT_RAND_S

#include "Route.h"
#include "Customer.h"
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <iterator>
#include <stdexcept>



namespace PSASP {



// ...
Route::Route (Customer &base):
    euclidean_length (0.0),
    base (base) {}



// ...
Route::Route (const Route &r):
    base (r.base) {
    this->euclidean_length = r.euclidean_length;
    this->path = r.path;
}



// ...
const Route &Route::operator= (const Route &r) {
    this->euclidean_length = r.euclidean_length;
    this->base = r.base;
    this->path = r.path;
    return *this;
}



// ...
bool Route::add (const Customer &c, double *length_delta) {
    if (this->path.size () < 3) {
        this->path.push_back (c);
        *length_delta = this->euclidean_length;
        this->recompute_length ();
        *length_delta = this->euclidean_length - *length_delta;
        return true;
    } else {
        *length_delta = 0;
        return false;
    }
}



// ...
bool Route::add (const Customer &c) {
    if (this->path.size () < 3) {
        this->path.push_back (c);
        this->recompute_length ();
        return true;
    } else
        return false;
}



// ...
void Route::remove (const Customer &c, double *length_delta) {
    this->path.remove (c);
    *length_delta = this->euclidean_length;
    this->recompute_length ();
    *length_delta = this->euclidean_length - *length_delta;
}



// ...
void Route::remove (const Customer &c) {
    this->path.remove (c);
    this->recompute_length ();
}



// ...
double Route::recompute_length () {
    double curr_dist = 0.0;
    Customer *curr = &base;
    
    for (std::list<Customer>::iterator it = this->path.begin ();   it != this->path.end ();   it++) {
        curr_dist += curr->distance (*it);
        curr = &(*it);
    }
    this->euclidean_length = curr_dist;
    return this->euclidean_length;
}



// ...
double Route::length () const {
    return this->euclidean_length;
}



// ...
Customer &Route::get_random_customer () throw (std::range_error) {
    if (this->path.empty ())
        throw std::range_error ("Cannot select and return any customer -- path is empty!");

    std::list<Customer>::iterator it = this->path.begin ();
    unsigned int tmp_rnd;
    
    rand_s (&tmp_rnd);
    std::advance (it, tmp_rnd % this->path.size ());
    
    return *it;
}



} // end namespace PSASP
